@@ -1,4 +1,5 @@
 from typing import Optional, Union, Any, Tuple, Iterable
+from typing_extensions import Final
 
 from mypy.nodes import (
     Expression, ComparisonExpr, OpExpr, MemberExpr, UnaryExpr, StarExpr, IndexExpr, LITERAL_YES,
@@ -11,10 +12,6 @@ from mypy.nodes import (
 )
 from mypy.visitor import ExpressionVisitor
 
-MYPY = False
-if MYPY:
-    from typing_extensions import Final
-
 # [Note Literals and literal_hash]
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
@@ -98,7 +95,7 @@ class _Hasher(ExpressionVisitor[Optional[Key]]):
         return ('Literal', e.value)
 
     def visit_str_expr(self, e: StrExpr) -> Key:
-        return ('Literal', e.value)
+        return ('Literal', e.value, e.from_python_3)
 
     def visit_bytes_expr(self, e: BytesExpr) -> Key:
         return ('Literal', e.value)
